# display for tetris.tk game

wm title . tetris

# ------ size and parameters
# n and m already passed to wish
set d 14
set w [expr $d+2]

# ------ board
frame .b -borderwidth 3 -relief sunken
pack .b -side top -fill both -expand true
canvas .b.board -bg white -width [expr $m*$w] -height [expr $n*$w]
pack .b.board -padx 10 -pady 10 -side top -fill none -expand true

# ------ controls
frame .d -borderwidth 5
pack .d -side bottom -fill x
frame .c -borderwidth 5
pack .c -side bottom -fill x
set count 0
proc write {x} {puts -nonewline stdout $x; flush stdout;}	
button .c.slow -text Slower -width 3 -command {write -}
button .c.fast -text Faster -width 3 -command {write +}
label .c.score -text Score -width 5
label .c.count -textvariable count -width 5 -bg white \
   -font lucidasans-bold-18
pack .c.slow .c.fast -side left -padx 3
pack .c.count .c.score -side right -padx 5

frame .d.q -borderwidth 1 -relief sunken
button .d.q.quit -text Quit -width 3 -command {write q}
button .d.new -text New -width 3 -command {write n}
button .d.help -text Help -width 3 -command {write h}
set status "Play"
proc Pause {} {global status; 
   if {$status=="Play"} {write r} else {write p}}
button .d.pause -textvariable status -width 3 -command Pause
pack .d.q -side right -padx 3
pack .d.q.quit -padx 3 -pady 3
pack .d.help .d.new .d.pause -side left -padx 3

# ------ clear and update-cell procedures
proc clear {} {global count; global status; \
               set count 0; set status "Play"
.b.board itemconfigure all -fill white}
proc updcir {x y c} {.b.board itemconfigure T$x.$y -fill $c}

# ------ create cells (all white)
proc cell {x y w} {\
     .b.board create rectangle\
        [expr [expr $y*$w]+2] [expr [expr $x*$w]+2]\
        [expr [expr $y*$w]+$w+1] [expr [expr $x*$w]+$w+1] \
        -outline {} -fill white -tag T$x.$y}
set x $n; set y $m
 while {$x >= 0} \
   {while {$y >= 0} {cell $x $y $w; incr y -1}; set y $m; incr x -1}

# ------ binding keys
bind all <Left> {write j}
bind all <Right> {write l}
bind all <Up> {write a}
bind all <Down> {write k}
bind all <space> {write { }}
bind all <Shift-space> {write z}
bind all <Control-Key-c> {write q}

# ------ help window
set helpmsg "\n\
Click on Play to start game.\n\n\
Moving pieces:\n\
\   Left arrow\tleft\n\
\   Right arrow\tright\n\
\   Down arrow\trotate\n\
\   Up arrow\tflip\n\
\   space\t\tdown fast\n\
\   Shift+space\tzap (cheat!)\n"

proc Help {} {
   toplevel .h 
   wm title .h help
   message .h.msg -textvariable helpmsg -justify left -bg white -font lucidasans-12
   button .h.ok -text OK -command {destroy .h; write H}
   pack .h.msg
   pack .h.ok -side right -padx 3 -pady 3
   }

