/* f-cells.h
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

Format layet: Cells layout.

Version 5.0; Jan 99;

*/




/* 1. definitions. ----------------------------------------------------	*/

/* m-options.h dfinit num, unum, byte, Min, Max */



/* 2. mots (words, ou contenus). --------------------------------------	*/

/* 2.1. types header, stel, word --------------------------------------	*/

typedef struct			/* headers (for all cells) */
   {
#ifdef BIG_ENDIAN_TARGET
    unsigned 
      copy: 1,  		/* copied bit				*/
      format: 1,  		/* 0: hunk, 1: vect			*/
      mutable: 1,		/* mutable				*/
      writable: 1,              /* copy-on-write			*/
      packing: 2,               /* size of packed units is		*/
				/* (2^packing) bytes (hunks)		*/
                                /* 0 + share-on-clone flag (vects)	*/
      unused: 2,                /* nbre d'octets inutilises (hunks)	*/
                                /* pruning marks in LCS (vects)		*/
      size:  24;  		/* taille cellule+header (en stels)	*/
#else BIG_ENDIAN_TARGET
    /* fields ordered in reverse order; eases big<->little conversion	*/
    unsigned 
      size:  24,
      unused: 2,
      packing: 2,
      writable: 1,
      mutable: 1,
      format: 1,
      copy: 1;
#endif BIG_ENDIAN_TARGET
  } header;             

typedef union stel		/* stel, quantum d'allocation		*/
     {
      header h;         	/* soit un header */
      union stel *p;      	/* soit un pointeur (ou valeur)		*/
     } *word;		/* type 'word': pointeur vers un stel ou valeur */

/* Un stel est soit un header, soit un paquet de caracteres / shorts /
long   / doubles, soit  un  pointeur vers  un stel  (qui  doit etre le
premier stel d'une cellule), soit une valeur. Dans le cas des valeurs,
le type  word est obtenu par  coercion. en propre: typedef union {stel
*ptr; num val;} word; mais words ne sont plus  alors des pointeurs, ce
qui est barbant ... */


/* 2.2. codage des valeurs et pointeurs:

Depuis V4.4,  un mot (word), ou contenu,  est soit une Valeur, soit un
Pointeur, soit Error.

Pointeurs: Representes par les pointeurs C correspondants, sans codage
(sauf type casting).  Les pointeurs LCS sont  toujours multiples de 4,
et  se terminent dont par 00.  Le mot 0x0 n'est  pas un pointeur, mais
MkPtr/IsPtr ne le testent pas;

Valeurs: La partie de l'espace virtuel non utilisee pour les pointeurs
sera utilisee pour coder un espace de valeurs. Les valeurs sont codees
comme des adresses "impaires", en decalant de 1 a gauche et en forcant
le dernier bit a` 1.

Error:  0x0 n'est ni  un pointeur, ni une valeur;  peut servir de code
d'exception.  Error must NOT be stored  in the heap   or stack when gc
occurs; Error obeys IsPtr test but points to illegal ...
*/


/* 2.3. macros pour manipuler les mots. -------------------------------	*/

/* Error: */

#define Error    	((word)0L) 	/* Error			*/
#define IsError(w)      (w==Error)


/* Valeurs: */

#define ZERO    	0x00000001L
#define UN      	0x00000003L
#define DEUX            0x00000005L

#define Zero		((word)ZERO)	/* le "zero" du dom de valeurs	*/
#define Un		((word)UN)	/* le "un" du dom de valeurs	*/
#define Deux            ((word)DEUX)

/* MkVal de'cale  gauche et force le bit 0 a` 1 */
#define MkVal(x)        (word)(((x)<<1) | ZERO)   /* entier -> valeur	*/
#define IsVal(x)        (((num)(x))&ZERO)         /* valeur ?           */
#define DeVal(x)        (((num)(x))>>1)           /* word -> num	*/


/* Pointeurs */

/* x assumed <> 0 */
#define MkPtr(x)        ((word)(x))       /* pointer -> word		*/
#define IsPtr(x)        (!IsVal(x))
/* to distinguish error from a ptr, IsPtr should be defined by:
   #define IsPtr(x)        (((num)(x)) && !((num)(x) & UN))		*/
#define DePtr(x)        ((word *)(x))     /* word -> pointer		*/




/* 3. Cellules (cells, ou contenants). --------------------------------	*/

/* Pas de  type "cellule"; une  cellule est identifiee par un pointeur
   vers sa premire composante. */

/* headers (macros partagees par toutes les formes de cellules): */

#define getFormat(x) 		(int)(((x)-1)->h.format)	
#define setFormat(x,f) 		{(((x)-1)->h.format)=f;}	
#define getCopy(x) 		(int)(((x)-1)->h.copy)	
#define setCopy(x,c) 		{(((x)-1)->h.copy)=c;}	
#define getMutable(x)		(int)(((x)-1)->h.mutable)	
#define isMutable(x)            (int)(getMutable(x))
#define setMutable(x,s)		{(((x)-1)->h.mutable)=s;}	
#define getWritable(x)		(int)(((x)-1)->h.writable)	
#define setWritable(x,s)	{(((x)-1)->h.writable)=s;}	
#define isWritable(x)           (int)(getWritable(x))
#define getPacking(x)		(int)(((x)-1)->h.packing)	
#define setPacking(x,s)   	{(((x)-1)->h.packing)=s;}	
#define getShare(x)		(int)(((x)-1)->h.packing)	
#define setShare(x,s)   	{(((x)-1)->h.packing)=s;}	
#define getUnused(x)		(int)(((x)-1)->h.unused)	
#define setUnused(x,s)   	{(((x)-1)->h.unused)=s;}	
#define getSize(x)		(int)(((x)-1)->h.size)  	
#define setSize(x,s)  		{(((x)-1)->h.size)=s;}	
#define getFullSize(x)		(getSize(x) + ((getFormat(x)) ? isMutable(x) : 0))
#define setHeader(x,h)          {((x)-1)->p=(h);}

