/* x-alloc.h
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

Foreign function interface; allocation macros;

Version 5.0.18; June 99;

*/






/* ====================================================================	*/
/* 1. stack allocation: -----------------------------------------------	*/
/* ====================================================================	*/

#define FlushStack()   	{topHold=hold;}
#define PushStack(w)   	{*topHold++=(w);}
#define PopStack()     	(*(--topHold))
#define TopStack()     	(*(topHold-1))
#define ShrinkStack(n) 	{topHold-=(n);}
#define NthStack(n)   	(*(topHold-(n)))
#define StackSize()    	(num)(topHold-hold)



/* ====================================================================	*/
/* 2. heap allocation ------------------------------------------------- */
/* ====================================================================	*/

extern int Muted;
extern int Moved;


/* space reservation -------------------------------------------------- */

#define Reserve0(n)	{if ((freeHeap+(n))>endToSpace) Collect2(n);}
#define Reserve1(n,w)	{if ((freeHeap+(n))>endToSpace) \
			   {PushStack(w); Collect2(n); w=PopStack();}}
#define Reserve2(n,w1,w2) { \
                         if ((freeHeap+(n))>endToSpace) \
			   {PushStack(w1); PushStack(w2); \
			    Collect2(n); \
			    w2=PopStack(); w1=PopStack();}}
#define Reserve3(n,w1,w2,w3) { \
                         if ((freeHeap+(n))>endToSpace) \
			   {PushStack(w1); PushStack(w2); PushStack(w3); \
			    Collect2(n); \
			    w3=PopStack(); w2=PopStack(); w1=PopStack();}}
#define Reserve4(n,w1,w2,w3,w4)	{ \
                         if ((freeHeap+(n))>endToSpace) \
			   {PushStack(w1); PushStack(w2); PushStack(w3); \
			    PushStack(w4); Collect2(n); w4=PopStack(); \
			    w3=PopStack(); w2=PopStack(); w1=PopStack();}}
#define Reserve5(n,w1,w2,w3,w4,w5) { \
                         if ((freeHeap+(n))>endToSpace) \
			   {PushStack(w1); PushStack(w2); PushStack(w3); \
                            PushStack(w4); PushStack(w5); Collect2(n); \
			    w5=PopStack(); w4=PopStack(); w3=PopStack(); \
                            w2=PopStack(); w1=PopStack();}}
#define Reservek(n,a,k)	{register int i; \
                         if ((freeHeap+(n))>endToSpace) \
                           {for (i=0; i<k; i++) PushStack(a[i]); \
			    Collect2(n); \
			    for (i=k-1; i>=0; i--) a[k]=PopStack();}}


/* preinitialized headers --------------------------------------------- */

#define HUNK		(word)0x00000000L
#define MHUNK		(word)0x20000000L
#define DOUBLE		(word)0x0c000003L
#define VECT		(word)0x40000000L
#define MVECT		(word)0x60000000L
#define SINGL		(word)0x40000002L
#define MSINGL		(word)0x60000002L
#define PAIR		(word)0x40000003L
#define MPAIR		(word)0x60000003L


/* Hunks;  indices 0..HunkSize-1 -------------------------------------- */
/* note: pas de verification de taille. Il est peu probable que 24 bits
   ne soient pas suffisant. Hunks sont au moins de taille 1 (non vrifi). */

/* generic: note: no "mutables" slot for mutable chunks */

#define IsChunk(x)  		(!(getFormat(x)))	/* x suppose ptr  */
#define IsChars(x)  		((IsChunk(x)) && (!getPacking(x)))
#define IsShorts(x)  		((IsChunk(x)) && (1==getPacking(x)))
#define IsLongs(x)  		((IsChunk(x)) && (2==getPacking(x)))
#define IsDoubles(x)  		((IsChunk(x)) && (3==getPacking(x)))

#define setCharsHeader(x,n)     {setHeader(x,HUNK); setSize(x,2+(((n)-1)>>2)); \
                                 setUnused(x,((getSize(x)-1)<<2)-(n));}


/* specializations: */

/* n in bytes */
#define setChars(r,h,n)         {setHeader(freeHeap,h); setSize(freeHeap,2+(((n)-1)>>2)); \
                                 setUnused(freeHeap,((getSize(freeHeap)-1)<<2)-(n)); \
				 r=freeHeap; freeHeap+=getSize(freeHeap);}
#define allocChars(r,n)         {setChars(r,HUNK,n);}
#define allocMChars(r,n)        {setChars(r,MHUNK,n);}
#define Chars(x)                ((char *)(x))
#define GetChar(x,n)		(Chars(x)[n])
#define SetChar(x,n,y)		{Chars(x)[n]=(y);}
#define CharsSize(x) 		(((getSize(x)-1)<<2)-getUnused(x))
extern word AllocChars(int);
extern word AllocMChars(int);

/* n in shorts */
#define setShorts(r,h,n)        {setHeader(freeHeap,h); setSize(freeHeap,2+(((n)-1)>>1)); \
                                 setUnused(freeHeap,((getSize(freeHeap)-1)<<1)-(n)); \
                                 setPacking(freeHeap,1); r=freeHeap; freeHeap+=getSize(freeHeap);}
#define allocShorts(r,n)	{setShorts(r,HUNK,n);}
#define allocMShorts(r,n)	{setShorts(r,MHUNK,n);}
#define Shorts(x)               ((short *)(x))
#define GetShort(x,n)  		(Shorts(x)[n])
#define SetShort(x,n,d)         {Shorts(x)[n]=(d);}
#define ShortsSize(x)           (((getSize(x)-1)<<1)-(getUnused(x)>>1)))
extern word AllocShorts(int);
extern word AllocMShorts(int);

/* n in longs */
#define setLongs(r,h,n)         {setHeader(freeHeap,h); setSize(freeHeap,(n)+1); \
                                 setPacking(freeHeap,2); r=freeHeap; freeHeap+=getSize(freeHeap);}
#define allocLongs(r,n)		{setLongs(r,HUNK,n);}
#define allocMLongs(r,n)	{setLongs(r,MHUNK,n);}
#define Longs(x)                ((long *)(x))
#define GetLong(x,n)  		(Longs(x)[n])
#define SetLong(x,n,d) 		{Longs(x)[n]=(d);}
#define LongsSize(x)            (getSize(x)-1)
extern word AllocLongs(int);
extern word AllocMLongs(int);

/* n in doubles */
#define setDoubles(r,h,n)       {setHeader(freeHeap,h); setSize(freeHeap,((n)<<1)+1); \
                                 setPacking(freeHeap,3); r=freeHeap; freeHeap+=getSize(freeHeap);}
#define allocDoubles(r,n)	{setDoubles(r,HUNK,n);}
#define allocMDoubles(r,n)	{setDoubles(r,MHUNK,n);}
#define Doubles(x)              ((double *)(x))
#define DoublesSize(x)          ((getSize(x)-1)>>1)
#ifdef ALIGN_DOUBLES
#define GetDoubles(x,n)  	({double D; \
			          *(int*)(&D)=GetLong(x,n); *(1+(int*)(&D))=GetLong(x,n+1); \
				  D;})
#define SetDoubles(x,n,d) 	{double D = d; \
				 SetLong(x,n,*(int*)(&D)); SetLong(x,n+1,*(1+(int*)(&D)));}
#else ALIGN_DOUBLES
#define GetDoubles(x,n)  	(Doubles(x)[n])
#define SetDoubles(x,n,d) 	{Doubles(x)[n]=(d);}
#endif ALIGN_DOUBLES
extern word AllocDoubles(int);
extern word AllocMDoubles(int);
extern word AllocAddr(void *);  /* stores a C-pointer */

/* specialization of Doubles, to store a single C-doubles: */
#define allocDouble(r,d)        {setHeader(freeHeap,DOUBLE); SetDouble(freeHeap,d); \
				 r=freeHeap; freeHeap+=3;}
#define IsDouble(w)             (*(DePtr(w))==DOUBLE)
#ifdef ALIGN_DOUBLES
#define GetDouble(x)  		({double D; \
			          *(int*)(&D)=GetLong(x,0); *(1+(int*)(&D))=GetLong(x,1); \
				  D;})
#define SetDouble(x,d) 		{double D = d; \
				 SetLong(x,0,*(int*)(&D)); SetLong(x,1,*(1+(int*)(&D)));}
#else ALIGN_DOUBLES
#define GetDouble(x)  		(*Doubles(x))
#define SetDouble(x,d) 		{*Doubles(x)=(d);}
#endif ALIGN_DOUBLES
extern word AllocDouble(double);


/* Vecteurs;  indices 1..VectSize ------------------------------------- */

/* Mutable vectors are followed by a slot used by the copier to link
all mutable cells. This link is NOT filled at allocation time, but its 
space must be reserved, so that copy will not overflow the current heap. 
Mutable hunks need not be linked. */ 

/* ---- generic: */

/* n = number of components */
#define setWords(r,n)           {setSize(freeHeap,1+(n)); r=freeHeap;}
#define allocWords(r,n)         {setHeader(freeHeap,VECT); setWords(r,n); \
                                 freeHeap+=getSize(freeHeap);}
#define allocMWords(r,n)        {setHeader(freeHeap,MVECT); setWords(r,n); \
                                 freeHeap+=1+getSize(freeHeap);}
extern word AllocWords(int);
extern word AllocMWords(int);

#define Words(x)		((word *)(x))
#define GetWord(x,n)     	(Words(x)[n])           /* indexed from 0 */
#define SetWord(x,n,y)  	{Words(x)[n]=(y);}      /* indexed from 0 */
#define IsWords(x) 		(getFormat(x))	  /* vecteur ?		     */
#define WordsSize(x) 		(getSize(x)-1)   /* taille, sans le header  */
#define LastWord(x)      	GetWord(x,WordsSize(x)-1)

#define setWordsHeader(x,n)     {setHeader(x,VECT); setSize(x,1+(n));}


/* specializations: */

/* ---- size 1 */
#define fillSingle(w1)          {SetWord(freeHeap,0,w1);}
#define setSingle(r,w1)         {fillSingle(w1); r=freeHeap;}
#define allocSingle(r,w1)       {setHeader(freeHeap,SINGL); setSingle(r,w1); freeHeap+=2;}
#define allocMSingle(r,w1)      {setHeader(freeHeap,MSINGL); setSingle(r,w1); freeHeap+=3;}
/* test, access, update: */
#define GetSingle(w)	        (GetWord(w,0))
#define SetSingle(w,c) 	        {SetWord(w,0,c);}
extern word AllocSingle(word);
extern word AllocMSingle(word);

/* ---- size 2 */
#define fillPair(w1,w2)         {fillSingle(w1); SetWord(freeHeap,1,w2);}
#define setPair(r,w1,w2)        {fillPair(w1,w2); r=freeHeap;}
#define allocPair(r,w1,w2)      {setHeader(freeHeap,PAIR); setPair(r,w1,w2); freeHeap+=3;}
#define allocMPair(r,w1,w2)     {setHeader(freeHeap,MPAIR); setPair(r,w1,w2); freeHeap+=4;}
extern word AllocPair(word, word);
extern word AllocMPair(word, word);

/* ---- size 3 */
#define fillTriple(w1,w2,w3)     {fillPair(w1,w2); SetWord(freeHeap,2,w3);}
#define setTriple(r,w1,w2,w3)    {setSize(freeHeap,4); fillTriple(w1,w2,w3); r=freeHeap;}
#define allocTriple(r,w1,w2,w3)  {setHeader(freeHeap,VECT); setTriple(r,w1,w2,w3); freeHeap+=4;}
#define allocMTriple(r,w1,w2,w3) {setHeader(freeHeap,MVECT); setTriple(r,w1,w2,w3); freeHeap+=5;}
extern word AllocTriple(word, word, word);
extern word AllocMTriple(word, word, word);

/* ---- size 4 */
#define fillQuad(w1,w2,w3,w4)     {fillTriple(w1,w2,w3); SetWord(freeHeap,3,w4);}
#define setQuad(r,w1,w2,w3,w4)    {setSize(freeHeap,5); fillQuad(w1,w2,w3,w4); r=freeHeap;}
#define allocQuad(r,w1,w2,w3,w4)  {setHeader(freeHeap,VECT); setQuad(r,w1,w2,w3,w4); freeHeap+=5;}
#define allocMQuad(r,w1,w2,w3,w4) {setHeader(freeHeap,MVECT); setQuad(r,w1,w2,w3,w4); freeHeap+=6;}
extern word AllocQuad(word, word, word, word);
extern word AllocMQuad(word, word, word, word);

/* ---- size 5 */
#define fillPenta(w1,w2,w3,w4,w5)     {fillQuad(w1,w2,w3,w4); SetWord(freeHeap,4,w5);}
#define setPenta(r,w1,w2,w3,w4,w5)    {setSize(freeHeap,6); fillPenta(w1,w2,w3,w4,w5); r=freeHeap;}
#define allocPenta(r,w1,w2,w3,w4,w5)  {setHeader(freeHeap,VECT); setPenta(r,w1,w2,w3,w4,w5); freeHeap+=6;}
#define allocMPenta(r,w1,w2,w3,w4,w5) {setHeader(freeHeap,MVECT); setPenta(r,w1,w2,w3,w4,w5); freeHeap+=7;}
extern word AllocPenta(word, word, word, word, word);
extern word AllocMPenta(word, word, word, word, word);

/* ---- size n (n>0) */
#define fillVect(r,fr,n)   {setSize(freeHeap,(n)+1); while ((n)--) SetWord(freeHeap,n,fr[n]); r=freeHeap;}
#define allocVect(r,fr,n)  {setHeader(freeHeap,VECT); fillVect(r,fr,n); freeHeap+=getSize(freeHeap);}
#define allocArray(r,fr,n) {setHeader(freeHeap,MVECT); fillVect(r,fr,n); freeHeap+=getSize(freeHeap)+1;}
extern word AllocVect(word [], int);
extern word AllocArray(word [], int);

