.TH LCS n "Version 5.2" "LCS system"
.SH NAME
lcs	   \- the lcs runtimes and user interfaces

.SH SYNOPSIS
.B lcs
[\-\f2n\f1one | \-\f2l\f1ight | \-\f2s\f1trong | \-\f2f\f1ile file] [\-\f2c\f1heck] [\-\f2h\f1elp]
[\-\f2m\f1emory statsz dynsz] [-\f2g\f1c  width thres] [\-\f2t\f1ime slice]
[\-\f2q\f1uiet | \-\f2d\f1iscreet | \-\f2v\f1erbose] [\-\f2r\f1egister host port]
[-\f2b\f1ackground] [-\f2p\f1rotect level] [-\f2a\f1rguments 'a1 ... an' | a1 ... an]

The flags may be abbreviated. Only the last occurrence of a flag is taken
into account.

.SH DESCRIPTION

Command lcs calls the lcs runtime system with its interactive user interface.

.SH OPTIONS
.TP
.B \-\f2n\f1one | \-\f2l\f1ight | \-\f2s\f1trong |\ \-\f2f\f1ile\ file

Selects a bytecode core image to be run and the appropriate runtime.

Options -(n|l|s) are shortands  for -f <APPSDIR>/lcs-(n|l|s), in which
<APPSDIR>  is   the  applications  directory  of   lcs  (determined  at
installation).   That is,  lcs  -n  calls the  version  of lcs  without
processes,  lcs  -l  calls   the  version  with  light-only  processes
(default), and  lcs -s  calls the version  with both light  and strong
processes.

Option -light (the default option) should be convenient for nearly all
purposes.  When processes are  not  needed, option -n  would  slightly
speeds garbage collection.  Option -s  should only be used when strong
processes   are needed   (rarely),    as it  significantly   slows LCS
interprocess communications.

Option -f \f2file\f1 takes \f2file\f1  as the core image to be loaded,
rather than the  default user interface.  If the file  is not found in
the current directory, then it is searched in <APPSDIR>. The file must
have been  created from lcs by  either the dump primitive  or the save
primitive.  Core files are  target independent: files produced on some
machine may be  read on any other.  If  \f2file\f1 is made executable,
then executing  "\f2file\f1 args"  is exactly equivalent  to executing
"lcs  args0  -f \f2file\f1  args",  in  which  args0 are  the  default
parameters for the application declared when the file was created (see
the  desription of  the dump  primitive in  the libraries  manual), or
empty if \f2file\f1 was created by the save primitive.

Default: -l

	
.TP
.B \-\f2c\f1heck

Does not run the core  specified by one  of the previous options,  but
just computes and prints statistics about its  contents, and checks its
consistency.  Can be used to  check that a  file is indeed an lcs core
file.  It also prints the default parameters assigned to the application
when it was created.


.TP
.B \-\f2h\f1elp

Recalls the syntax of the lcs command, with a short description of its
options.


.TP
.B \-\f2m\f1emory statsz dynsz

Heap  sizing parameters.   LCS uses  a generational  two-space copying
compacting  garbage  collector   with  dynamic  heap  re-sizing.   The
parameters of  flag m  determine dynamically the  size of  the working
space.   The  nominal space  size  is  (statsz  + dynsz*(size  of  old
generation)).  stasz is in kilobytes, dynsz is a nonnegative float.

Default:  -m 1024 2.0


.TP
.B \-\f2g\f1c width thres

Garbage collection and resizing parameters, both in ratios in terms of
nominal  heap size. Parameter width determine  an  interval of working
sizes.  After a  major GC, a   resizing is performed if  the available
size overflows     or  underflows the    interval.   Parameter  thres
determines the occupancy  of the  working size   above which a   major
collection   is initiated.  A major  GC  is initiated  when the available
space falls below   (nominal/thres).  width and thres  must be floats
greater than 1.0.

Flag -g should not be used in normal practice. The default values have
been tuned  up  after measures on a   number of  typical applications;
changes  may result  in strong degradations  of garbage collection
performances.

Default:  -g 1.2 3.0


.TP
.B \-\f2t\f1ime slice

Sets the  length of the time  slice for the  time-slicing mechanism in
lcs runtimes supporting processes.  If \f2slice\f1 is 0, then time slicing is
disabled.  Otherwise, the maximum time  an LCS process may run without
being interrupted  is \f2slice\f1 milliseconds.  This option  is useless with
process option -none.

Default:	-t 500


.TP
.B \-\f2q\f1uiet | \-\f2d\f1iscreet | \-\f2v\f1erbose

Set verbosity level. 

-quiet makes  the runtime remain silent  in  all circumstances, except
fatal errors.  It should  be used when running standalone applications
to avoid confusion between messages issued by  lcs and those issued by
the application.

-discreet makes lcs print its banner, its leaving message, and the few
warnings and messages the runtime might produce on the lcs console.

-verbose sets a trouble-shooting mode.  The runtime may print a lot of
information relevant  to its own  internal operation.  In addition, it
may dynamically  perform a  number  of consistency checks.   This flag
should not be  used in normal circumstances;   It is intended to  help
locate bugs in the runtime systems.

Default:	-d


.TP
.B \-\f2r\f1egister host port

Host and port number to be used to search the LCS services database in
distributed  applications. An  empty  host parameter  defaults to  the
local host.

The host  and port  passed by -register  supersede the  default server
host and port number specified  at installation of lcs. An alternative
to the use  of flag -r is to set  environment variables LCS_SERVER and
LCS_PORTNO.  If  one of  these variable is  defined, then  its content
supersedes the default value of the relevant parameter.

Default: -r "" 4500


.TP
.B \-\f2b\f1ackground

Runs  the lcs  command in  the background.  To launch applications not
requiring user interaction, or providing their own interaction means.


.TP
.B \-\f2p\f1rotect level

Sets the  bottom protection  level for all  processes run in  this lcs
application.   Each protection  level enables  a subset  of bytecodes.
Level  0  enables  all  bytecodes.   Level 1  is  recommended  to  run
applications with untrusted  contents, it prevents confidentiality and
integrity losses on  the host environment.  Level 2  is recommended to
run  untrusted imported  code into  host applications,  it  forbids in
addition   to   use   primitives    that   could   affect   the   host
application. Protection level may also  be increased by programs, on a
"per-process"  basis,  by  primitive  System.Protect.  Level  0  means
unprotected.

Default: 	-p 0


.TP
.B \-\f2a\f1rguments 'a1 ... an', or a1 ... an

These  arguments  are passed to lcs  or  a standalone lcs application,
from which they can be retrieved by arg_list(), formatted as a list of
strings. Without the -a  flag, the  a1 ...  an list should  follow the
sequence of command options, if any. Flag  -a allows to pass arguments
beginning with "-"; the arguments  passed by -a must  all appear in the
same quote pair (if more than  one arg are passed,  e.g. lcs -a "hello
345" is equivalent to lcs hello 345).

Default: 	-a ""



.SH AUTHORS
Bernard Berthomieu, with contributions by Didier Giralt, Jean-Paul Gouyon, Thierry Le Sergent, LAAS/CNRS, 1988-2000.

