(* alist.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

association lists.

*)




(* signature

structure Alist =
   exception Assoc : exn
   val assoc = - : ''a -> (''a * 'b) list -> 'b
   exception Rev_assoc : exn
   val rev_assoc = - : ''a -> ('b * ''a) list -> 'b
   exception Chop_find : exn
   val chop_find = - : ('a -> bool) -> 'a list -> 'a list * 'a list
   val drop = - : ''a -> ''a list -> ''a list
   exception Replace : exn
   val replace = - : ''a -> ''a -> ''a list -> ''a list

*)




structure Alist = (* association lists *)
struct

exception Assoc;
fun assoc x =
  let fun rassoc [] = raise Assoc
        | rassoc ((y,z)::t) = if x=y then z else rassoc t
  in rassoc
  end;
fun assoc x [] = raise Assoc | assoc x ((y,z)::t) = if x=y then z else assoc x t;

exception Rev_assoc;

fun rev_assoc x =
  let fun rassoc [] = raise Rev_assoc
        | rassoc ((y,z)::t) = if x=z then y else rassoc t
  in rassoc
  end;

fun rev_assoc x [] = raise Rev_assoc
  | rev_assoc x ((y,z)::t) = if x=z then y else rev_assoc x t;

(* l1, (h::l2) such that h = find p l *)
exception Chop_find;
fun chop_find p [] = raise Chop_find
  | chop_find p (h::t) =
	if p h then ([], h::t)
        else (let val (l1,l2) = chop_find p t in ((h::l1), l2) end);

(* replaces first # of i by v in e *)
exception Replace;
fun replace i v e =
        let val (l1,(x::l2)) = chop_find (fn x => x = i) e
        in (l1 @ v::l2)
        end;

(* drop first # of i from e *)
fun drop i e =
        let val (l1,(x::l2)) = chop_find (fn x => x = i) e
        in l1 @ l2
        end;

end; (* structure *)




