(* bytearray.sml
 _______________________________________________________________________
|                                                                       |
|                             Projet LCS                                |
|                         Groupe OLC, CNRS/LAAS                         |
|                        7, Av. du Colonel Roche                        |
|                          31077 Toulouse Cedex                         |
|_______________________________________________________________________|
 
byte arrays
 
*)
 
 
 
 
(* interface
 
type bytearray
exception Subscript : exn
exception Size : exn
val bytearray : int * int -> bytearray
val bytearray2 : int -> bytearray
val length : bytearray -> int
val sub : bytearray * int -> int
val update : bytearray * int * int -> unit

val bytearrayofstring : string -> bytearray
val stringofbytearray : bytearray -> string

val tabulate : int * (int -> int) -> bytearray
val bytearrayoflist : int list -> bytearray
val listofbytearray : bytearray -> int list

*)



(* implementation *)

structure Bytearray =
struct
open Core.Bytearray;
local
fun mka [e] n = bytearray (n+1,e)
  | mka (h::t) n = 
	let val a = mka t (n+1) in update (a,n,h); a end;
in
fun bytearrayoflist [] = bytearray (0,0)
  | bytearrayoflist l = mka l 0;
end;
fun listofbytearray a =
	case length a of
	  0 => []
	| n => let fun loba k 0 = []
		     | loba k n = sub (a,k)::loba (k+1) (n-1)
	       in loba 0 n
	       end;
fun bytearrayofstring "" = bytearray (0,0)
  | bytearrayofstring s =
	let val n = string_length s
	    val a = bytearray (n,0)
	    fun mka 0 _ = a
	      | mka n k = (update (a,k,ord (substring(s,k,1)));
			   mka (n-1) (k+1))
	in mka n 0
	end;
fun stringofbytearray b =
	implode (List.map chr (listofbytearray b));
end;





(* notes

bytes are represented by integers in the range 0..255.
(functions above take integers modulo 256)
 
*)

