(* codec.lcs
 _______________________________________________________________________
|                                                                       |
|                             Projet LCS                                |
|                         Groupe OLC, CNRS/LAAS                         |
|                        7, Av. du Colonel Roche                        |
|                          31077 Toulouse Cedex                         |
|_______________________________________________________________________|

various encoders/decoders for arbitrary bases (<255)
symbols are 1-character in \000 to \255
 
*)
 
 
(* signature:
 
structure Codec =
   val mk_codec : string -> (int -> string) * (string -> int)
   val d'b : string -> int
   val b'd : int -> string
   val d'h : string -> int
   val h'd : int -> string
   val d'o : string -> int
   val o'd : int -> string
   val bools'byte : int -> bool array
   val bytes'int  : int -> int array
*)
 
 
 
(* various encoders/decoders for arbitrary bases (<255) *)
(* symbols are 1-character in \000 to \255 *)


needs "tlist"; 	(* nth, etc *)
needs "alist"; 	(* chop_find, etc *)

structure Codec =
struct

local
open Tlist
open Alist
in
fun mk_codec repr = (* codec generator *)
	(* arg repr determine base (=size of) and representation.
	   e.g. repr = "01" makes a binary encoder/decoder;
		repr = "0123456789abcdef" makes an hexa encoder/decoder *)
let val repr = explode repr
    val base = length repr
    fun encode d = nth repr (d+1)
    and decode h = length (fst (chop_find (fn x => x=h) repr))
    fun E x = (* decimal to base *)
	let val X = ref (abs x)
	    and L = ref (tl [""])
	in while !X >= base
	   do let val (q,r) = divide (!X,base)
	      in L := encode r:: !L; X := q
	      end
	 ; implode (encode (!X):: !L)
	end
    fun D x = (* base to decimal *)
	let val L = ref (explode x)
	    val d = ref 0
	in while not (null (!L))
	   do let val (h::t) = !L
	      in d := base * !d + decode h; L := t
	      end
         ; !d
	end
in (E,D)
end;
end;

val (b'd,d'b) = mk_codec "01";			(* birary codecs *)
val (o'd,d'o) = mk_codec "01234567";		(* octal codecs	 *)
val (h'd,d'h) = mk_codec "0123456789abcdef";	(* hexa codecs	 *)


local
open Array
in
fun bytes'int n = (* int -> 4 bytes *)
	let val b = array (4,0)
	    fun explode n i =
		if i=0 orelse n=0 then update (b,i,n)
		else let val (q,r) = divide (n,256)
		     in (update (b,i,r); explode q (i-1))
		     end
	in explode (abs n) 3; b
	end;

fun bools'byte n = (* byte -> 8 bools *)
	let val b = array (8,false)
	    fun explode n i =
		if i=0 orelse n=0 then update (b,i,n>0)
		else let val (q,r) = divide (n,2)
		     in (update (b,i,r>0); explode q (i-1))
		     end
	in explode (abs n) 7; b
	end;
end;

end;




(* e.g.
- b'd 256;
"100000000"
- d'h "61a80";
val it = 400000 : int
- o'd it;
val it = "1415200" : string

- open Array;
- bytes'int 46565633;
val it = [|2, 198, 137, 1|] : int array
- bools'byte (Array.sub (it,1));
val it = 
[|true, true, false, false, false, true, true, false|] : bool array
*)

