(* io.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

string input/output on files and windows

*)




(* interface

type instream
type outstream
type control
type window

exception Io : string -> exn

(* streams: *)

val std_in : instream
val open_in : string -> instream
val close_in : instream -> unit
val getc : instream -> string
val input : instream * int -> string
val input_line : instream -> string
val lookahead : instream -> string
val end_of_stream : instream -> bool
val is_term_in : instream -> bool
val de_instream : instream -> dev

val std_out : outstream
val open_out : string -> outstream
val open_append : string -> outstream
val open_excl : string -> outstream
val close_out : outstream -> unit
val puts : outstream * string * int -> unit
val output : outstream * string -> unit
fun output_line : outstream * string -> unit
val flush_out : outstream -> unit
val is_term_out : outstream -> bool
val de_outstream : outstream -> dev

val std_ctrl : control
val wait_ctrl : control -> string
val de_control : control -> dev

(* windows: *)

val std_win : window
val open_window : string * string * int -> window

val close_win : window -> unit
val win_in : window -> instream
val win_out : window -> outstream
val win_ctrl : window -> control
val win_frame : window -> dev
val win_term : window -> string
val input_win : window * int -> string
val input_line_win : window -> string
val output_win : window * string -> unit
val output_line_win : window * string -> unit
val wait_ctrl_win : window -> string
val de_window : window -> control * instream * outstream * dev * string

val open_set_win : string * (int * int) * (int * int) * string * string -> int -> window
val open_win : unit -> window
val open_xterm : unit -> window

*)





(* abstraction:
abstype instream = I of dev		(* input streams 	*)
and    outstream = O of dev		(* output streams	*)
and      control = C of dev		(* control streams	*)
and    errstream = E of dev		(* error streams	*)
and       window = W of 		(* windows		*)
		      dev 		(* control 		*)
		    * dev 		(* instream		*)
		    * dev 		(* outstream		*)
		    * dev 		(* frame device		*)
		    * string		(* terminfo string	*)
with

fun open_in f = I (dopen (f,0));
fun input (I d,n) = gets (d,n);
...... etc,
end;
*)



(* implementation: *)

structure Io =
struct
open Core.Io;
fun is_term_in ii = Core.Dev.is_term (de_instream ii);
fun is_term_out oo = Core.Dev.is_term (de_outstream oo);
local
fun cmd (title,(xpos,ypos),(lin,col),font,disp) =
	  (* open_window assumes double quotes properly paired *)
	  "xterm -T " ^ "\"" ^ title ^ "\"" ^
	  (case font of "" => "" | _ => " -fn " ^ font) ^
	  " -g " ^ string_of_int col ^ "x" ^ string_of_int lin ^ "+" ^
		   string_of_int xpos ^ "+" ^ string_of_int ypos ^
	  (case disp of "" => "" | _ => " -display " ^ disp) ^
	  " -e "     (* remainder appended by open_window primitive *)
val count = ref 0;
fun default () =
	 (count := 1 + !count;
	  ("lcs"^string_of_int(!count),(20+rand 100,20+rand 100),(25,80),"",""));
in
fun open_set_win parms ld = open_window (cmd parms,"xterm",ld);
fun open_win () = open_set_win (default ()) 1;
val open_xterm = open_win;
end;
end;


