(* queue.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

queues (functional).

*)




(* interface

structure Queue =

   abstype 'a queue
   exception Deq : exn
   val empty   : 'a queue
   val isempty : 'a queue -> bool
   val enq     : 'a -> 'a queue -> 'a queue
   val deq     : 'a queue -> 'a * 'a queue
   val head    : 'a queue -> 'a

   abstype 'a iqueue
   val newqueue     : unit -> '1a iqueue
   val clearqueue   : 'a iqueue -> unit
   val isemptyqueue : 'a iqueue -> bool
   val enqueue      : 'a -> 'a iqueue -> unit
   val dequeue      : 'a iqueue -> 'a
   val headqueue    : 'a iqueue -> 'a
 
   type 'a oqueue = (unit -> unit) *
		    (unit -> bool) *
		    ('a -> unit) * 
                    (unit -> 'a) *
		    (unit -> 'a)
   val newoqueue = - : unit -> '1a ?.oqueue

*)




(* implementation *)

structure Queue =
struct

abstype 'a queue = Q of ('a list * 'a list)
with
val empty = Q ([],[]);
fun isempty (Q ([],[])) = true | isempty _ = false;
fun enq x (Q (front,rear)) = Q (front,x::rear);
exception Deq;
fun deq (Q (hd::tl,rear)) = (hd,Q(tl,rear))
  | deq (Q ([],rear)) = deq (Q (rev rear,[]))
  | deq _ = raise Deq;
fun head (Q ([],[])) = raise Deq
  | head (Q (hd::tl,_)) = hd
  | head (Q (_,rear)) = hd (rev rear);
end;

abstype 'a iqueue = I of 'a queue ref		(* queues with states *)
with 
fun newqueue () = I (ref empty);
fun clearqueue (I r) = r := empty;
fun isemptyqueue (I r) = isempty (!r);
fun enqueue (I r) x = r := enq x (!r);
fun dequeue (I r) = let val (x,s) = deq (!r) in r := s; x end;
fun headqueue (I r) = head (!r);
end;

type 'a oqueue = 				(* object style queues *)
		(unit -> unit) *		(* clear method	*)
		(unit -> bool) *		(* isempty 	*)
		('a -> unit) *			(* enqueue	*)
		(unit -> 'a) *			(* dequeue	*)
		(unit -> 'a);			(* head		*)
fun newoqueue () : 'a oqueue =
	let val r = ref empty
	in (fn () => r := empty,
	    fn () => isempty (!r),
	    fn x => r := enq x (!r),
	    fn () => let val (x,s) = deq (!r) in r := s; x end,
	    fn () => head (!r))
	end;

end; (* structure *)


