(* rat.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

rational numbers (Daniel Loeb, july 90)

*)




(* signature 

type rat
con // : int * int -> rat
val over : int * int -> rat
val ~ : rat -> rat
val + : rat * rat -> rat
val - : rat * rat -> rat
val * : rat * rat -> rat
val / : rat -> rat -> rat
exception Inv;
val inv : rat -> rat
val eq : rat * rat -> bool
val > : rat * rat -> bool
val < : rat * rat -> bool
val >= : rat * rat -> bool
val <= : rat * rat -> bool

val real'rat : rat -> real
val rat'real : real -> rat
val rat2'real : int -> real -> rat

val string_of_rat : rat -> string
exception Rat_of_string : exn
val rat_of_string : string -> rat
val print : rat -> rat
val makestring : rat -> string
*)



(* implementation *)

needs "alist"; (* chop_find *)


structure Rat =
struct

structure Rep = struct
datatype rat = // of int * int;
end;
infix 8 //;
open Rep;

overload ~ + - * / > < >= <=;
infix 8 over;
infix 7 * /;
infix 6 + -;

local
fun gcd n 0 = n | gcd n m = gcd m (n mod m);
fun zero (a//_) = a = 0;
fun pos (a//_) = a > 0;
in
fun n over d = let val g = if d>=0 then abs(gcd n d) else ~ (abs(gcd n d))
	       in  (n div g)//(d div g)
	       end;
fun ~ (n//d) = (~n)//d;
fun (n1//d1) + (n2//d2) = (n1*d2 + d1*n2) over (d1*d2);
fun (x:rat) - y = x + (~ y);
exception Inv;
fun inv (_//0) = raise Inv | inv (n//d) = d//n;
fun (n1//d1) * (n2//d2) = (n1*n2) over (d1*d2);
fun x / y = x * (inv y);
infix 4 eq;
fun x eq y = zero (x - y);
fun x >  (y:rat) = pos (x-y);
fun x <  (y:rat) = y > x;
fun x >= (y:rat) = not (x < y);
fun x <= (y:rat) = not (x > y);
end;
nonoverload ~ + - * / > < >= <=;



(* rat<->real conversions *)
fun real'rat (x//y) = (real x) / (real y);
local
fun continued x 0 = nil
  | continued x n =
	let val n0 = floor x
	    val x0 = real n0
	in if x0 = x then [n0] else n0 :: (continued (1.0/(x-x0)) (n-1))
					  handle floor => []
	end;
fun rat'c (n::_) 1 = n//1
  | rat'c (n::ns) k = (n//1 + inv (rat'c ns (k-1)))
  | rat'c _ 0 = 0//1;
fun dic (ns,min,max,curr,result) = (* ovf detection and dichotomy *)
      let val res = rat'c ns curr
          val nxt = (curr+max) div 2
      in if nxt=curr then res
         else (* try to improve *) dic (ns,curr,max,nxt,res)
      end
    handle Overflow => 
      let val nxt = (min + curr) div 2
      in if nxt=curr then result
         else (* reduce *) dic (ns,min,curr,nxt,result)
      end;
fun rat1'c ns = (* with automatic precision adjustement *)
      let val k = length ns in dic (ns,0,k,k,1//0) end;
fun rat2'c nil    = 0//1
  | rat2'c (n::nil) = n//1
  | rat2'c (n::ns)  = (n//1 + inv (rat2'c ns));
in
fun rat'real x = (* automatic precision *)
      rat1'c (continued x 100);
fun rat2'real n x = (* precision n explicit *)
      rat2'c (continued x n);
end;
 

fun string_of_rat (x//y) = string_of_int x^"//"^string_of_int y;
exception Rat_of_string;
fun rat_of_string s =
	let val (l1,l2) = Alist.chop_find (fn x => x ="/") (explode s)
		handle Alist.Chop_find => raise Rat_of_string
	in case l2 of
	  ("/"::"/"::l) => (int_of_string (implode l1) // int_of_string (implode l)
		handle Int_of_string => raise Rat_of_string)
	| _ => raise Rat_of_string
	end;

val makestring = string_of_rat;
fun print r = (output (std_output(), string_of_rat r); r);

end; (* structure *)



(* usage:
overload ~ + - * / > < >= <= print makestring;
open Rat;
nonoverload ~ + - * / > < >= <= print makestring;
infix 8 // over;
infix 4 eq;
*)
