(* real.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

*)






(* interface

type real
exception Real : string -> exn
val ~ : real -> real
val + : real * real -> real
val - : real * real -> real
val * : real * real -> real
val / : real * real -> real
val < : real * real -> bool
val > : real * real -> bool
val <= : real * real -> bool
val >= : real * real -> bool
val floor : real -> int
val truncate : real -> int
val round : real -> int
val real : int -> real
val abs : real -> real
val pow : real * real -> real
val sin : real -> real
val cos : real -> real
val arctan : real -> real
exception Sqrt : exn
val sqrt : real -> real
val exp : real -> real
val ln : real -> real
val min : real * real -> real
val max : real * real -> real
val pow : real * real -> real
val mathlib : int * real -> real;
val pi : real
val e : real
val rrand : unit -> real
val string_of_real : real -> string
val real_of_string : string -> real
val print : real -> real
val makestring : real -> string

*)



(* implementation *)

(* primitive mathlib (i,r) provides f r for i:
 1 sin		 2 cos		 3 tan
 4 asin		 5 acos		 6 atan
 7 sinh		 8 cosh		 9 tanh
10 asinh 	11 acosh 	12 atanh
13 sqrt 	14 cbrt 
15 ln 		16 exp *)


structure Real =
struct
infix 6 + -;
infix 7 * / pow;
infix 4 < > <= >=;
open Core.Real;
fun min (x:real,y) = if x<y then x else y;
fun max (x:real,y) = if x<y then y else x; 
fun truncate (r:real) = (* truncation *)
	if r>=0.0 then floor r else Core.Int.~ (floor (~r));
fun round (r:real) = (* nearest integer *)
	if r-real(floor r) >= 0.5 then Core.Int.+ (floor r,1) else floor r;
exception Sqrt;
fun sin r = mathlib (1,r)
and cos r = mathlib (2,r)
and arctan r = mathlib (6,r)
and sqrt r = mathlib (13,r) handle Real _ => raise Sqrt;
exception Ln and Exp;
fun ln r = mathlib (15,r) handle Real _ =>  raise Ln
and exp r = mathlib (16,r) handle Real _ => raise Exp;
val pi = 4.0 * (arctan 1.0);      (* 3.14159265358979.. *)
val e = exp 1.0;		  (* 2.71828182845905.. *)
fun print r = (write (string_of_real r); r);
val makestring = string_of_real;
end;



 

