(* stree.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

Binary search trees with string keys.

*)



(* signature

structure Stree =
  abstype 'a stree
  val nullstree : 'a stree
  exception Lookupstree : exn
  val lookupstree : 'a stree -> string -> 'a
  val updatestree : 'a stree * (string * 'a) -> 'a stree
  val streedepth : 'a stree -> int
  val balancestree : 'a stree -> 'a stree
  val list_of_stree : 'a stree -> (string * 'a) list
  val mapstree : ('a -> 'b) -> 'a stree -> 'b stree

*)


needs "tlist";  (* chop, etc *)


structure Stree =
struct

abstype 'a stree = (* with strings keys, could be parameterized *)
          null
        | stree of string * 'a * 'a stree * 'a stree

with
val nullstree = null;
exception Lookupstree;
fun lookupstree tr k = (* returns content in tr for key k *)
        let fun lookup null = raise Lookupstree
              | lookup (stree (K,C,tr1,tr2)) =
                        if k>K then lookup tr2
                        if K>k then lookup tr1
                        else C
        in lookup tr
        end;
fun updatestree (tr,kc as (k,c)) = (* extends or updates tr with kc *)
        (* updates content of key k if exists, otherwise creates entry *)
        let fun update null = stree (k,c,null,null)
              | update (stree (K,C,tr1,tr2)) =
                        if k>K then stree (K,C,tr1,update tr2)
                        if K>k then stree (K,C,update tr1,tr2)
                        else stree (K,c,tr1,tr2)
        in update tr
        end;
fun list_of_stree null = [] (* get entries list in alpha order of keys *)
  | list_of_stree (stree (k,c,tr1,tr2)) =
			list_of_stree tr1 @ (k,c)::list_of_stree tr2;
fun balancestree tr = (* balances search tree tr, so that depth is minimal *)
    let fun balance l = case l of
          [] => null
        | [(s,c)] => stree (s,c,null,null)
        | l => let val (l1,l2 as (s,c)::t2) = Tlist.chop ((length l) div 2) l
               in stree (s,c,balance l1,balance t2)
               end
    in balance (list_of_stree tr)
    end;
fun streedepth null = 0 (* depth of deepest item in stree *)
  | streedepth (stree(_,_,e1,e2)) = max (1 + streedepth e1, 1 + streedepth e2);


fun mapstree f null = null
  | mapstree f (stree (k,c,tr1,tr2)) =
	stree (k,f c,mapstree f tr1, mapstree f tr2)

end; (* abstype *)

(* strees with states
*** TO BE PROVIDED ***
*)

end; (* structure *)






