(* string.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

*)






(* interface

type string
exception Ord : exn
val ord : string -> int
exception Chr : exn
val chr : int -> string

val > : string * string -> bool
val < : string * string -> bool
val >= : string * string -> bool
val <= : string * string -> bool
val string_length : string -> int
val size : string -> int
val ^ : string * string -> string
exception Size : exn	(* string size overflow *)
val implode : string list -> string
val implode_sep : string -> string list -> string
val explode : string -> string list
val explode_sep : string -> string -> string list
exception Substring : exn
val substring : string * int * int -> string
val print : string -> string
val makestring : string -> string

*)



(* equivalence

exception Size
datatype string = int list  (* in essence, but implemented more efficiently *)
fun ord s = < primitive>
fun chr n = <primitive>
fun op > (s1,s2) = <lexicographic ordering, primitive>
fun implode l = <primitive>
fun explode s = <primitive>
fun string_length s = length (explode s)
infix 6 ^
fun op ^ (s1, s2) = implode (s1 @ s2)
*)



(* implementation *)


structure String =
struct
infix 4 < > <= >=;
infix 6 ^;
open Core.String;
fun print s = (write s; s);
fun makestring (s:string) = s;
val size = string_length;   (* for sml compatibility *)
(* implodes, adding sep between strings *)
fun implode_sep sep [] = ""
  | implode_sep sep [x] = x
  | implode_sep sep (h::t) = h^sep^implode_sep sep t;
fun explode_sep sep s =
        let fun curr m n =
                if sep = substring (s,n,1)
                then if Int.> (n-m,0)
		     then substring (s,m,n-m) :: curr (n+1) (n+1)
		     else curr (n+1) (n+1)
                else curr m (n+1)
                handle Substring => [substring (s,m,n-m+1)]
        in case s of "" => [] | _ => curr 0 0
        end;
end;







