(* system.sml
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

*)






(* interface

exception Interrupt : exn
exception Bind : exn
exception Match : exn

datatype 'a option [''a option]
con None : 'a option
con Some : 'a -> 'a option

val sleep : real -> unit

val protect : int -> int

val lib_path : unit -> string list
val add_lib_path : string -> string list
val rem_lib_path : string -> string list
val init_lib_path : unit -> string list

val read_file : (string * bool * bool * bool) -> unit
val include : string -> unit
val use : string -> unit
val needs : string -> unit

val app_path : unit -> string list
val add_app_path : string -> string list
val rem_app_path : string -> string list
val init_app_path : unit -> string list

exception Dump : string -> exn
val dump : string * string -> unit
val save : string -> unit
exception Rest : string -> exn
val restore : string -> unit
val load : string -> unit

val clean : unit -> unit
val exit : unit -> unit
val quit : unit -> unit

val reboot : (unit -> unit) -> unit
(* reserved word in lcs -l and lcs -s *)
val kill : unit -> unit
(* reserved word in lcs -l and lcs -s *)

val lcs_print_init : unit -> unit
val lcs_pretty_printing : bool -> unit
val lcs_print_depth : int -> unit
val lcs_print_width : int -> unit

val argv : string vector
val arg_list : unit -> string list
val version : unit -> string * real * int
val runtime : unit -> (int * int)
val stats : Unit -> int * Int * int * int * int

val time : ('a -> 'b) -> 'a -> 'b
val timer : bool -> unit
val timers : bool -> unit
val print_code : bool -> bool

val gc : unit -> unit
val pp : unit -> unit

val prompt : (string * string) -> unit
val balance : unit -> unit
val depth : unit -> int list * int list
val extended_charset : bool -> unit
val factorize_types : bool -> unit
val balance_types : bool -> unit
val refutable_lazy_patts : bool -> unit

exception Malloc : exn
exception Cfun : string -> exn
val cfun_register : string * string * string -> string array

*)



(* implementation *)

(* Prelude.String assumed loaded *)

structure System =
struct

open Core.System;

datatype 'a option = None | Some of 'a;

local
fun strip s = String.substring(s,0,String.size s-1);
in
fun init_lib_path () = (* initializes lib search path *)
	 (List.map rem_lib_path (lib_path ())
	 ; add_lib_path "."
	 ; case Core.Vector.sub (argv,10) of
	  "" => ()
	| libpath =>
		let val (ii,oo) = execute ("ls "^libpath)
		    fun ls "" = [] | ls s = strip s :: ls (input_line ii)
		in add_lib_path (strip libpath)
		 ; List.map (fn s => add_lib_path (libpath^s)) (ls (input_line ii))
		 ; close_in ii; close_out oo
		end
	; lib_path ());
fun init_app_path () = (* initializes app search path *)
	  (List.map rem_app_path (app_path ())
	 ; add_app_path "."
	 ; case Core.Vector.sub (argv,9) of
	  "" => app_path ()
	| apppath => add_app_path (strip apppath));
end;

fun include f = (* mandatory reading, prints dots *)
	read_file (f,false,false,false);
fun use f     = (* mandatory reading, prints results *) 
	read_file (f,false,true,false);
fun needs f   = (* conditional reading, prints dots *)
	read_file (f,false,false,true);

fun arg_list () = String.explode_sep " " (Core.Vector.sub (argv,12));

(* gets available run/gc times and stats, and formats them *)
fun time f x =
        let val (r0,g0) = runtime ()
            and (M0,m0,a0,p0,s0) = stats ()
            val y = f x
            val (r,g) = runtime ()
            and (M1,m,a,p,s) = stats ()
	    fun write s = Core.Io.output (Core.Io.std_out,s)
        in write (Core.Real.string_of_real (real (r-r0) / 1000.0)); write "s " (* run time *)
         ; write (Core.Real.string_of_real (real (g-g0) / 1000.0)); write "s " (* gc time  *)
         ; write (Core.Int.string_of_int (a-a0)); write "kb ("   (* space allocated      *)
         ; write (Core.Int.string_of_int (M1-M0)); write "+"     (* major gc count       *)
         ; write (Core.Int.string_of_int (m-m0)); write ")gc "   (* minor gc count       *)
         ; write "\n"
         ; y
        end;

end;



