#!/bin/sh
# run this script to launch lcs

# lcs installed from config information inserted below:
# ------ begin config info ------
# ------ end config info ------

# launching:

# default rts and application:
RTS=$EXEDIR/rts-l
APP=$SHADIR/apps/lcs-l

# default values for parameters
WSIZE=1024      # static nominal heap size (kb)
WFZEN=2.0       # dynamic nominal heap size (% of old)
WWIDTH=1.2      # width of free interval in heap
WTHRES=3.0      # minor/major gc threshold
VERBOSE=1       # verbosity: quiet(0), verbose(1), or debug(2)
TSLICE=500      # half time slice (ms; useless if -none)
PROTECT=0	# protection level

# initial values:
app=$APP
rts=$RTS
verbose=$VERBOSE
wsize=$WSIZE
wfzen=$WFZEN
wwidth=$WWIDTH
wthres=$WTHRES
protect=$PROTECT
timeslice=$TSLICE
execdir=$EXEDIR/
appsdir=$SHADIR/apps/
libsdir=$SHADIR/lib/
if [ x"$SERVER" = x ]; then server=`hostname`.`domainname`; else server=$SERVER; fi
portno=$PORTNO
arglist=
bground=0

usage() {
  echo "Usage: lcs [-none|-light|-strong|-file core] [-check] [-help]"
  echo "           [-quiet|-discreet|-verbose] [-memory kb fl] [-gc fl fl]"
  echo "           [-timeslice ms] [-register host port] [-background]"
  echo "           [-protect level] [-arguments \"a1 ... an\"|a1 ... an]"
}

help()
{
  usage
  echo "arguments (flags may be abbreviated):"
  echo "FLAGS, ARGS                      NOTES                       DEFAULT"
  echo "-help                            this mode                   "
  echo "-none|-light|-strong|-file core  core to be loaded           -light"
  echo "-check                           just check core, don't run  "
  echo "-quiet|-discreet|-verbose        verbosity level             -discreet"
  echo "-protect int                     security level (>= 0)       -protect 0"
  echo "-memory int fl                   heap sizing parameters      -m $WSIZE $WFZEN"
  echo "-gc fl fl                        garbage coll. parameters    -g $WWIDTH $WTHRES"
  echo "-timeslice ms                    time slice (useless if -n)  -t $TSLICE"
  echo "-background                      runs lcs in background      "
  if [ x"$SERVER" = x ]
  then
  echo "-register host port              lcsd registration service   -r \"\" $PORTNO"
  else
  echo "-register host port              lcsd registration service   -r $SERVER $PORTNO"
  fi
  echo "-arguments a1 ... an|a1 ... an   args passed to application  "
}

while [ x"$1" != x ]; do
    case $1 in
        # rts and application:

	-n | -none)
            app=$SHADIR/apps/lcs-n
	    rts=$EXEDIR/rts-n
	    shift
	    continue;;

	-l | -light)
            app=$SHADIR/apps/lcs-l
	    rts=$EXEDIR/rts-l
	    shift
	    continue;;

	-s | -strong)
            app=$SHADIR/apps/lcs-s
            rts=$EXEDIR/rts-s
	    shift
	    continue;;

	-f | -file) 
	    # locate file and set app
	    if [ -f $2 ]
            then
	       app=`pwd`/$2
	    else
	       if [ -f $SHADIR/apps/$2 ]
	       then
	          app=$SHADIR/apps/$2
	       else
	          echo "can't find file $2"
	          exit 1
	       fi
            fi
	    # set rts
            case `tail +3 $app` in 
               \#lcs-n*) rts=$EXEDIR/rts-n;;
               \#lcs-l*) rts=$EXEDIR/rts-l;;
               \#lcs-s*) rts=$EXEDIR/rts-s;;
               *) echo "$2 is not an lcs application"
                  exit 1;;
            esac
	    shift
	    shift
	    continue;;

	-c | -check)
            rts=$EXEDIR/rts-c
	    shift
	    break;;

	-h | -help)
	    help
	    exit 0;;

	# parameters

	-q | -quiet)
            verbose=0
	    shift
	    continue;;

	-d | -discreet)
            verbose=1
	    shift
	    continue;;

	-v | -verbose)
            verbose=2
	    shift
	    continue;;

	-p | -protect)
            protect=$2
	    if [ $2 -ge 0 ]
	    then 
	       shift
	       shift
	       continue
	    else
	       echo "$2 bad value for -protect (int >= 0)"
	       exit 1
	    fi;;

	-m | -memory)
	    wsize=$2
            wfzen=$3
	    # if [ $2 -ge 1 -a `echo "$3 >= 0.0" | bc` = 1 ]
	    if [ $2 -ge 1 -a x`echo "if ($3 >= 0.0) 1" | bc` = x1 ]
	    then
	       shift
	       shift
	       shift
	       continue
            else
               echo "bad value(s) for -memory (int >= 1; float >= 0.0)"
	       exit 1
            fi;;

	-g | -gc)
            wwidth=$2
	    wthres=$3
	    # if [ `echo "$2 >= 1.1" | bc` = 1 -a `echo "$3 >= 1.1" | bc` = 1 ]
	    if [ x`echo "if ($2 >= 1.1) 1" | bc` = x1 -a x`echo "if ($3 >= 1.1) 1" | bc` = x1 ]
	    then
	       shift
	       shift
	       shift
	       continue
            else
               echo "bad value(s) for -gc (float >= 1.1; float >= 1.1)"
	       exit 1
            fi;;

        -t | -timeslice)
            tslice=$2
	    if [ $2 -ge 0 ]
	    then
	       shift
	       shift
	       continue
            else
               echo "bad value(s) for -timeslice (int >= 0)"
	       exit 1
            fi;;

	-b | -background)
            bground=1
	    shift
	    continue;;

	-r | -register)
	    server=$2
	    if [ x"$2" = x ]; then server=; else server=$2; fi
            portno=$3
	    if [ $3 -ge 0 ]
	    then
	       portno=$3
	    else
	       echo "bad value(s) for registration port (int >= 0)"
	       exit 1
            fi
	    shift
	    shift
	    shift
	    continue;;

	-a | -arguments)
	    arglist=$2
	    shift
	    shift
	    continue;;

	-*)
	    usage
	    exit 1;;

	*)
            # everything left is passed to application
	    arglist="$arglist $@"
	    break;;
    esac
done


if [ "$bground" = 1 ]
then
exec $rts $app $wsize $wfzen $wwidth $wthres $verbose $timeslice $execdir $appsdir $libsdir "" "$arglist" $server $portno $protect &
else
exec $rts $app $wsize $wfzen $wwidth $wthres $verbose $timeslice $execdir $appsdir $libsdir "" "$arglist" $server $portno $protect
fi

